;*******************************************************************
;*           GRAY SCALE LIBRARY FOR TI-86 VERSION 1.0              *
;*                 Written by Robert Niedziela                     *
;*                             1998                                *
;*-----------------------------------------------------------------*
;* This is the library of basic routines for gray scale graph mode *
;* Version for 4 levels of gray without double buffering           *
;*-----------------------------------------------------------------*
;* There is variable UserCounter which is incremented every        *
;* interrupt - use it if you need any counter.                     *
;*******************************************************************

#define PAGE1_BPL1         $3C
#define PAGE1_BPL2         $0A


_user_int_ram EQU $d2fd                 ;Area where interrupt handler goes..

;+--------------------------------------------------------------------------+
;| Open gray scale graph mode                                               |
;+--------------------------------------------------------------------------+
;| Parameters: none                                                         |
;+--------------------------------------------------------------------------+
;| Returns: nothing                                                         |
;+--------------------------------------------------------------------------+
;| Registers destroyed: HL, DE, BC, A                                       |
;+--------------------------------------------------------------------------+
;| Comments: Iinitialize interrupt routine and init variables               |
;+--------------------------------------------------------------------------+
OpenGray:
        ;store 6 bytes following _plotSScreen
        ld      hl, _plotSScreen+1018
        ld      de, _plotSScreen
        ld      bc, 6
        ldir

        res     2,(iy+$23)                      ;turn user int off so it won't get called by
                                                ;accident before we're ready
        ld      hl,IntRoutine                   ;copy prog to user int buffer
        ld      de,_user_int_ram+1
        ld      bc,InterruptMemEnd-IntRoutine   ;length of handler
        ldir                                    ;hl is the source, de is the destination,
                                                ;and bc is the length

        ld      a,(_user_int_ram+1)             ;set up checksum byte
        ld      hl,_user_int_ram+($28*1)        ;load byte
        add     a,(hl)                          ;add to total
        ld      hl,_user_int_ram+($28*2)        ;load byte
        add     a,(hl)                          ;add to total
        ld      hl,_user_int_ram+($28*3)        ;load byte
        add     a,(hl)                          ;add to total
        ld      hl,_user_int_ram+($28*4)        ;load byte
        add     a,(hl)                          ;add to total
        ld      hl,_user_int_ram+($28*5)        ;load byte
        add     a,(hl)                          ;add to total
        ld      (_user_int_ram),a               ;load checksum in its location

        xor     a                               ;init variables
        ld      (IntCounter), a
        ld      (UserCounter), a

        ld      de, IntData                     ;init page addrs
        ld      hl, SourceIntData
        ld      bc, 2
        ldir

        set     2,(iy+$23)                      ;turn it on
        ret


;+--------------------------------------------------------------------------+
;| Close gray scale graph mode                                              |
;+--------------------------------------------------------------------------+
;| Parameters: none                                                         |
;+--------------------------------------------------------------------------+
;| Returns: nothing                                                         |
;+--------------------------------------------------------------------------+
;| Registers destroyed: A, BC, DE, HL                                       |
;+--------------------------------------------------------------------------+
;| Comments: After closing gray scale graph mode it clears bitplanes        |
;|           and sets LCD to display standard memory area ($FC00)           |
;+--------------------------------------------------------------------------+
CloseGray:
        res     2, (iy+$23)                       ;disable the jump to $d2fd
        ld      a, $3C
        out     (0), a
        ;restore 6 bytes following _plotSScreen
        ld      de, _plotSScreen+1018
        ld      hl, _plotSScreen
        ld      bc, 6
        ldir
        ;clear  LCD and Plot memory
        call    ClearPage
        ret


;+--------------------------------------------------------------------------+
;| Clear graph's memory                                                     |
;+--------------------------------------------------------------------------+
;| Parameters: none                                                         |
;+--------------------------------------------------------------------------+
;| Returns: nothing                                                         |
;+--------------------------------------------------------------------------+
;| Registers destroyed: BC, DE, HL                                          |
;+--------------------------------------------------------------------------+
;| Comments: none                                                           |
;+--------------------------------------------------------------------------+
ClearPage:
        ld      hl, (Pages)
        call    Clear1024
        ld      hl, (Pages+2)
        call    Clear1024
        ret
Clear1024:
        ld      d, h
        ld      e, l
        inc     de
        ld      (hl), 0
        ld      bc, 1024
        ldir
        ret


;+--------------------------------------------------------------------------+
;| Wait for VBL interrupt                                                   |
;+--------------------------------------------------------------------------+
;| Parameters: none                                                         |
;+--------------------------------------------------------------------------+
;| Returns: nothing                                                         |
;+--------------------------------------------------------------------------+
;| Registers destroyed: A                                                   |
;+--------------------------------------------------------------------------+
;| Comments: This routine waits for VBL interrupt and exits.                |
;|           Call it to reduce flicking while changing pages.               |
;+--------------------------------------------------------------------------+
WaitForVBL:
        push    af
WaitForVBLInternal:
        ld      a,(IntCounter)
        or      a
        jr      nz, WaitForVBLInternal
        pop     af
        ret


;---------------------------------------------------------------------
;| This is the interrupt routine                                     |
;---------------------------------------------------------------------
IntRoutine:
        push    af
        push    hl

        in      a,(3)
        bit     1,a           ; check that it is a vbl interrupt
        jr      z,EndIntRoutine

        ld      hl, UserCounter
        inc     (hl)
        ld      hl, IntData

        ld      a,(IntCounter)
        cp      5
        jr      z,DisplayLast
        cp      3
        jr      c,DisplayBpl1
        jr      DisplayBpl2

DisplayBpl1:                            ;Dislpay 1st bitplane
        inc     a
        ld      (IntCounter),a
        ld      a, (hl)
        out     (0),a
        jr      EndIntRoutine

DisplayBpl2:                            ;Display 2nd bitplane
        inc     a
        ld      (IntCounter),a
        inc     hl
        ld      a, (hl)
        out     (0),a
        jr      EndIntRoutine

DisplayLast:                            ;Display 1st bitplane again
        ld      a, (hl)
        out     (0), a
        xor     a
        ld      (IntCounter),a

EndIntRoutine:
        pop     hl
        pop     af
        ret
InterruptMemEnd:

IntCounter              equ _user_int_ram+InterruptMemEnd-IntRoutine+2
IntData                 equ _user_int_ram+InterruptMemEnd-IntRoutine+3
UserCounter             equ _user_int_ram+InterruptMemEnd-IntRoutine+5



SourceIntData:
.db PAGE1_BPL1, PAGE1_BPL2

;addresses of ram area for each bitplane
Pages:
.dw VIDEO_MEM, _plotSScreen+6

.END
